'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ArtifactFile = require('./ArtifactFile.js');

var _ArtifactFile2 = _interopRequireDefault(_ArtifactFile);

var _Multipart = require('./Multipart.js');

var _Multipart2 = _interopRequireDefault(_Multipart);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_NETWORK = "IPFS";
var SUPPORTED_TYPES = ["Audio", "Video", "Image", "Text", "Software", "Web", "Research", "Property"];

var CHOP_MAX_LEN = 370;
var FLODATA_MAX_LEN = 528;

module.exports = function () {
	function Artifact(input) {
		_classCallCheck(this, Artifact);

		this.artifact = {
			floAddress: "",
			info: {},
			details: {},
			storage: { network: DEFAULT_NETWORK, files: [] },
			payment: {}
		};

		this.FileObjects = [];
		this.Multiparts = [];
		this.fromMultipart = false;

		if (input) {
			// If we are being passed in an array, it might be multiparts so try to load from that
			if (Array.isArray(input) && input.length > 1 && input[0] instanceof _Multipart2.default) {
				this.fromMultipart(input);
			} else if (typeof input === "string") {
				try {
					this.fromJSON(JSON.parse(input));
				} catch (e) {}
			} else if ((typeof input === 'undefined' ? 'undefined' : _typeof(input)) === "object") {
				this.fromJSON(input);
			}
		}
	}

	_createClass(Artifact, [{
		key: 'setTXID',
		value: function setTXID(txid) {
			this.txid = txid;
		}
	}, {
		key: 'getTXID',
		value: function getTXID() {
			return this.txid;
		}
	}, {
		key: 'setPublisherName',
		value: function setPublisherName(pubName) {
			this.publisherName = pubName;
		}
	}, {
		key: 'getPublisherName',
		value: function getPublisherName() {
			return this.publisherName;
		}
	}, {
		key: 'setMainAddress',
		value: function setMainAddress(address) {
			this.artifact.floAddress = address;
		}
	}, {
		key: 'getMainAddress',
		value: function getMainAddress() {
			return this.artifact.floAddress;
		}
	}, {
		key: 'setTimestamp',
		value: function setTimestamp(time) {
			this.artifact.timestamp = time;
		}
	}, {
		key: 'getTimestamp',
		value: function getTimestamp() {
			return this.artifact.timestamp;
		}
	}, {
		key: 'setTitle',
		value: function setTitle(title) {
			this.artifact.info.title = title;
		}
	}, {
		key: 'getTitle',
		value: function getTitle() {
			return this.artifact.info.title;
		}
	}, {
		key: 'setDescription',
		value: function setDescription(description) {
			this.artifact.info.description = description;
		}
	}, {
		key: 'getDescription',
		value: function getDescription() {
			return this.artifact.info.description;
		}
	}, {
		key: 'setType',
		value: function setType(type) {
			type = this.capitalizeFirstLetter(type);

			if (SUPPORTED_TYPES.indexOf(type) === -1) {
				return "Type Not Supported!";
			}

			this.artifact.type = type;
		}
	}, {
		key: 'getType',
		value: function getType() {
			return this.artifact.type;
		}
	}, {
		key: 'setSubtype',
		value: function setSubtype(subtype) {
			subtype = this.capitalizeFirstLetter(subtype);

			this.artifact.subtype = subtype;
		}
	}, {
		key: 'getSubtype',
		value: function getSubtype() {
			return this.artifact.subtype;
		}
	}, {
		key: 'setYear',
		value: function setYear(year) {
			this.artifact.info.year = year;
		}
	}, {
		key: 'getYear',
		value: function getYear() {
			return this.artifact.info.year;
		}
	}, {
		key: 'setNSFW',
		value: function setNSFW(nsfwToggle) {
			this.artifact.info.nsfw = nsfwToggle;
		}
	}, {
		key: 'getNSFW',
		value: function getNSFW() {
			return this.artifact.info.nsfw || false;
		}
	}, {
		key: 'setTags',
		value: function setTags(tags) {
			if (Array.isArray(tags)) {
				this.artifact.info.tags = tags;
			} else {
				if (tags.split(",").length > 1) {
					this.artifact.info.tags = tags.split(",");
				} else {
					this.artifact.info.tags = [tags];
				}
			}
		}
	}, {
		key: 'getTags',
		value: function getTags() {
			return this.artifact.info.tags;
		}
	}, {
		key: 'setDetail',
		value: function setDetail(detailNode, info) {
			if (typeof info === "string" && info.split(",").length > 1) {
				this.artifact.details[detailNode] = info.split(",");
			} else {
				this.artifact.details[detailNode] = info;
			}

			this.artifact.details[detailNode] = info;
		}
	}, {
		key: 'getDetail',
		value: function getDetail(detailNode) {
			return this.artifact.details[detailNode];
		}
	}, {
		key: 'setSignature',
		value: function setSignature(signature) {
			this.artifact.signature = signature;
		}
	}, {
		key: 'getSignature',
		value: function getSignature() {
			return this.artifact.signature;
		}
	}, {
		key: 'setNetwork',
		value: function setNetwork(network) {
			if (network === "ipfs") network = "IPFS";

			this.artifact.storage.network = network;
		}
	}, {
		key: 'getNetwork',
		value: function getNetwork() {
			return this.artifact.storage.network;
		}
	}, {
		key: 'setLocation',
		value: function setLocation(location) {
			this.artifact.storage.location = location;
		}
	}, {
		key: 'getLocation',
		value: function getLocation() {
			return this.artifact.storage.location;
		}
	}, {
		key: 'setPaymentFiat',
		value: function setPaymentFiat(fiat) {
			this.artifact.payment.fiat = fiat;
		}
	}, {
		key: 'getPaymentFiat',
		value: function getPaymentFiat() {
			return this.artifact.payment.fiat;
		}
	}, {
		key: 'setPaymentScale',
		value: function setPaymentScale(newScale) {
			this.artifact.payment.scale = newScale;
		}
	}, {
		key: 'getPaymentScale',
		value: function getPaymentScale() {
			//	Check if scale is a string
			// 		If so, check if the string is a number, or represented as a ratio
			// 			return the parsed number or ratio bound
			if (this.artifact.payment.scale) {
				if (typeof this.artifact.payment.scale === "string") {
					if (isNaN(this.artifact.payment.scale) && this.artifact.payment.scale.split(":").length === 2) {
						return this.artifact.payment.scale.split(":")[1];
					} else if (!isNaN(this.artifact.payment.scale)) {
						return parseInt(this.artifact.payment.scale);
					}
				}

				return this.artifact.payment.scale;
			} else {
				// Return 1:1 scale if undefined! The user should ALWAYS set scale ON PUBLISH if they are using a scale!
				return 1;
			}
		}
	}, {
		key: 'setSuggestedTip',
		value: function setSuggestedTip(sugTipArray) {
			this.artifact.payment.tips = sugTipArray;
		}
	}, {
		key: 'getSuggestedTip',
		value: function getSuggestedTip() {
			return this.artifact.payment.tips;
		}
	}, {
		key: 'addTokenRule',
		value: function addTokenRule(tokenRule) {
			this.artifact.payment.tokens.push(tokenRule);
		}
	}, {
		key: 'getTokenRules',
		value: function getTokenRules() {
			return this.artifact.payment.tokens;
		}
	}, {
		key: 'addSinglePaymentAddress',
		value: function addSinglePaymentAddress(coin, address) {
			var tmpObj = {};
			tmpObj[coin] = address;

			if (!this.artifact.payment.addresses) this.artifact.payment.addresses = [];

			this.artifact.payment.addresses.push(tmpObj);
		}
	}, {
		key: 'getPaymentAddresses',
		value: function getPaymentAddresses() {
			return this.artifact.payment.addresses;
		}
	}, {
		key: 'setMultiwalletAddress',
		value: function setMultiwalletAddress(address) {
			this.artifact.payment.shortMW = address;
		}
	}, {
		key: 'getMultiwalletAddress',
		value: function getMultiwalletAddress() {
			return this.artifact.payment.shortMW;
		}
	}, {
		key: 'addSupportedMWCoin',
		value: function addSupportedMWCoin(coin) {
			if (!this.artifact.payment.coins) this.artifact.payment.coins = [];

			this.artifact.payment.coins.push(coin);
		}
	}, {
		key: 'getSupportedMWCoins',
		value: function getSupportedMWCoins() {
			return this.artifact.payment.coins;
		}
	}, {
		key: 'setRetailerCut',
		value: function setRetailerCut(newCut) {
			this.artifact.payment.retailer = newCut;
		}
	}, {
		key: 'getRetailerCut',
		value: function getRetailerCut() {
			return this.artifact.payment.retailer;
		}
	}, {
		key: 'setPromoterCut',
		value: function setPromoterCut(newCut) {
			this.artifact.payment.promoter = newCut;
		}
	}, {
		key: 'getPromoterCut',
		value: function getPromoterCut() {
			return this.artifact.payment.promoter;
		}
	}, {
		key: 'setMaxDiscount',
		value: function setMaxDiscount(newMax) {
			this.artifact.payment.maxdisc = newMax;
		}
	}, {
		key: 'getMaxDiscount',
		value: function getMaxDiscount() {
			return this.artifact.payment.maxdisc;
		}
	}, {
		key: 'addFile',
		value: function addFile(file) {
			if (file instanceof _ArtifactFile2.default) {
				this.FileObjects.push(new _ArtifactFile2.default(file.toJSON(), this));
			} else if ((typeof file === 'undefined' ? 'undefined' : _typeof(file)) === "object" || typeof file === "string") {
				this.FileObjects.push(new _ArtifactFile2.default(file, this));
			}
		}
	}, {
		key: 'getFiles',
		value: function getFiles() {
			return this.FileObjects;
		}
	}, {
		key: 'getThumbnail',
		value: function getThumbnail() {
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = this.getFiles()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var file = _step.value;

					if (file.getType() === "Image" && file.getSubtype() === "Thumbnail") {
						return file;
					}
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}

			return undefined;
		}
	}, {
		key: 'getDuration',
		value: function getDuration() {
			var _iteratorNormalCompletion2 = true;
			var _didIteratorError2 = false;
			var _iteratorError2 = undefined;

			try {
				for (var _iterator2 = this.getFiles()[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
					var file = _step2.value;

					if (!isNaN(file.getDuration())) {
						return file.getDuration();
					}
				}
			} catch (err) {
				_didIteratorError2 = true;
				_iteratorError2 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion2 && _iterator2.return) {
						_iterator2.return();
					}
				} finally {
					if (_didIteratorError2) {
						throw _iteratorError2;
					}
				}
			}

			return undefined;
		}
	}, {
		key: 'isValid',
		value: function isValid() {
			if (!this.artifact.info.title || this.artifact.info.title === "") {
				return { success: false, error: "Artifact Title is a Required Field" };
			}
			if (!this.artifact.floAddress || this.artifact.floAddress === "") {
				return { success: false, error: "floAddress is a Required Field! Please define it or Login!" };
			}

			return { success: true };
		}
	}, {
		key: 'isPaid',
		value: function isPaid() {
			var files = this.getFiles();

			var paid = false;

			if (files) {
				var _iteratorNormalCompletion3 = true;
				var _didIteratorError3 = false;
				var _iteratorError3 = undefined;

				try {
					for (var _iterator3 = files[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
						var file = _step3.value;

						if (file.isPaid()) paid = true;
					}
				} catch (err) {
					_didIteratorError3 = true;
					_iteratorError3 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion3 && _iterator3.return) {
							_iterator3.return();
						}
					} finally {
						if (_didIteratorError3) {
							throw _iteratorError3;
						}
					}
				}
			}

			return paid;
		}
	}, {
		key: 'toJSON',
		value: function toJSON() {
			this.artifact.storage.files = [];

			var _iteratorNormalCompletion4 = true;
			var _didIteratorError4 = false;
			var _iteratorError4 = undefined;

			try {
				for (var _iterator4 = this.FileObjects[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
					var file = _step4.value;

					this.artifact.storage.files.push(file.toJSON());
				}
			} catch (err) {
				_didIteratorError4 = true;
				_iteratorError4 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion4 && _iterator4.return) {
						_iterator4.return();
					}
				} finally {
					if (_didIteratorError4) {
						throw _iteratorError4;
					}
				}
			}

			var retJSON = {
				oip042: {
					artifact: this.artifact
				}
			};

			if (this.txid) {
				retJSON.txid = this.txid;
			}
			if (this.publisherName) {
				retJSON.publisherName = this.publisherName;
			}

			return JSON.parse(JSON.stringify(retJSON));
		}
	}, {
		key: 'fromJSON',
		value: function fromJSON(artifact) {
			if (artifact) {
				if (artifact.txid) {
					this.setTXID(artifact.txid);
				}
				if (artifact.publisherName) {
					this.setPublisherName(artifact.publisherName);
				}

				if (artifact['media-data']) {
					if (artifact['media-data']['alexandria-media']) {
						return this.importAlexandriaMedia(artifact['media-data']['alexandria-media']);
					} else {
						return { success: false, error: "No Artifact under Version!" };
					}
				} else if (artifact['oip-041']) {
					if (artifact['oip-041'].signature) {
						this.setSignature(artifact['oip-041'].signature);
					}

					if (artifact['oip-041'].artifact) {
						return this.import041(artifact['oip-041'].artifact);
					} else {
						return { success: false, error: "No Artifact under Version!" };
					}
				} else if (artifact.oip042) {
					if (artifact.oip042.signature) {
						this.setSignature(artifact.oip042.signature);
					}
					if (artifact.oip042.artifact) {
						return this.import042(artifact.oip042.artifact);
					} else if (artifact.oip042.publish && artifact.oip042.publish.artifact) {
						return this.import042(artifact.oip042.publish.artifact);
					} else if (artifact.oip042.floAddress) {
						// @TODO: Remove this once OIPd is fixed!
						// Returned info is Malformed!!! Remove this code!
						return this.import042(artifact.oip042);
					} else {
						return { success: false, error: "No Artifact under Version!" };
					}
				} else {
					return { success: false, error: "Artifact is Not a Supported Version!", detail: artifact };
				}
			} else {
				return { success: false, error: "Artifact Not Provided!" };
			}
		}
	}, {
		key: 'toString',
		value: function toString() {
			return JSON.stringify(this.toJSON());
		}
	}, {
		key: 'importAlexandriaMedia',
		value: function importAlexandriaMedia(artifact) {
			if (artifact.publisher) {
				this.setMainAddress(artifact.publisher);
			}
			if (artifact.timestamp) {
				this.setTimestamp(artifact.timestamp);
			}
			if (artifact.type) {
				var type = artifact.type;

				if (type === "music") {
					type = "Audio";
				}
				if (type === "book") {
					type = "Text";
					this.setSubtype("Book");
				}
				if (type === "thing") {
					type = "Web";
				}

				this.setType(type);
			}
			if (artifact.torrent) {
				this.setLocation(artifact.torrent);

				if (artifact.torrent.split(":")[0] === "btih") {
					this.setNetwork("bittorrent");
				}
			}
			if (artifact.info) {
				if (artifact.info.title) {
					this.setTitle(artifact.info.title);
				}
				if (artifact.info.description) {
					this.setDescription(artifact.info.description);
				}
				if (artifact.info.year) {
					this.setYear(artifact.info.year);
				}

				if (artifact.info['extra-info']) {
					var tmpFiles = [];
					var hadFiles = false;
					for (var key in artifact.info['extra-info']) {
						if (artifact.info['extra-info'].hasOwnProperty(key)) {
							if (key === "tags") {
								this.setTags(artifact.info['extra-info'][key]);
							} else if (key === "Bitcoin Address") {
								this.addSinglePaymentAddress("btc", artifact.info['extra-info'][key]);
							} else if (key === "DHT Hash") {
								var hash = artifact.info['extra-info'][key];

								this.setLocation(hash);
							} else if (key === "filename") {
								if (artifact.info['extra-info'][key] !== "none") tmpFiles.push({ fname: artifact.info['extra-info'][key] });
							} else if (key === "posterFrame") {
								tmpFiles.push({ fname: artifact.info['extra-info'][key], type: "Image", subtype: "Thumbnail" });
							} else if (key === "runtime") {
								this.setDetail("duration", artifact.info['extra-info'][key]);
							} else if (key === "files") {
								var fileList = artifact.info['extra-info'][key];

								var _iteratorNormalCompletion5 = true;
								var _didIteratorError5 = false;
								var _iteratorError5 = undefined;

								try {
									for (var _iterator5 = fileList[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
										var file = _step5.value;

										this.addFile(file);
									}
								} catch (err) {
									_didIteratorError5 = true;
									_iteratorError5 = err;
								} finally {
									try {
										if (!_iteratorNormalCompletion5 && _iterator5.return) {
											_iterator5.return();
										}
									} finally {
										if (_didIteratorError5) {
											throw _iteratorError5;
										}
									}
								}

								if (this.FileObjects.length > 0) hadFiles = true;
							} else {
								this.setDetail(key, artifact.info['extra-info'][key]);
							}
						}
					}

					if (!hadFiles) {
						var _iteratorNormalCompletion6 = true;
						var _didIteratorError6 = false;
						var _iteratorError6 = undefined;

						try {
							for (var _iterator6 = tmpFiles[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
								var file = _step6.value;

								this.addFile(file);
							}
						} catch (err) {
							_didIteratorError6 = true;
							_iteratorError6 = err;
						} finally {
							try {
								if (!_iteratorNormalCompletion6 && _iterator6.return) {
									_iterator6.return();
								}
							} finally {
								if (_didIteratorError6) {
									throw _iteratorError6;
								}
							}
						}
					}
				}
			}
			if (artifact.payment) {
				// if (artifact.payment.type && artifact.payment.type === "tip"){
				// 	this.setPaymentFiat(artifact.payment.fiat);
				// }
				// if (artifact.payment.scale){
				// 	this.setPaymentScale(artifact.payment.scale);
				// }
				// if (artifact.payment.sugTip){
				// 	this.setSuggestedTip(artifact.payment.sugTip)
				// }
			}
		}
	}, {
		key: 'import041',
		value: function import041(artifact) {
			if (artifact.publisher) {
				this.setMainAddress(artifact.publisher);
			}
			if (artifact.timestamp) {
				this.setTimestamp(artifact.timestamp);
			}
			if (artifact.type) {
				if (artifact.type.split("-").length === 2) {
					var type = artifact.type.split("-")[0];
					var subtype = artifact.type.split("-")[1];

					this.setType(type);
					this.setSubtype(subtype);
				} else if (artifact.type.split("-").length === 1) {
					this.setType(artifact.type);
				}
			}
			if (artifact.info) {
				if (artifact.info.title) {
					this.setTitle(artifact.info.title);
				}
				if (artifact.info.description) {
					this.setDescription(artifact.info.description);
				}
				if (artifact.info.year) {
					this.setYear(artifact.info.year);
				}
				if (artifact.info.tags) {
					this.setTags(artifact.info.tags);
				}
				if (artifact.info.nsfw) {
					this.setNSFW(artifact.info.nsfw);
				}

				if (artifact.info.extraInfo) {
					for (var key in artifact.info.extraInfo) {
						if (artifact.info.extraInfo.hasOwnProperty(key)) {
							this.setDetail(key, artifact.info.extraInfo[key]);
						}
					}
				}
			}

			if (artifact.storage) {
				if (artifact.storage.network) {
					this.setNetwork(artifact.storage.network);
				}
				if (artifact.storage.location) {
					this.setLocation(artifact.storage.location);
				}
				if (artifact.storage.files) {
					var _iteratorNormalCompletion7 = true;
					var _didIteratorError7 = false;
					var _iteratorError7 = undefined;

					try {
						for (var _iterator7 = artifact.storage.files[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
							var file = _step7.value;

							this.addFile(file);
						}
					} catch (err) {
						_didIteratorError7 = true;
						_iteratorError7 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion7 && _iterator7.return) {
								_iterator7.return();
							}
						} finally {
							if (_didIteratorError7) {
								throw _iteratorError7;
							}
						}
					}
				}
			}

			if (artifact.payment) {
				if (artifact.payment.fiat) {
					this.setPaymentFiat(artifact.payment.fiat);
				}
				if (artifact.payment.scale) {
					this.setPaymentScale(artifact.payment.scale);
				}
				if (artifact.payment.sugTip) {
					this.setSuggestedTip(artifact.payment.sugTip);
				}
				if (artifact.payment.tokens && Array.isArray(artifact.payment.tokens)) {
					var _iteratorNormalCompletion8 = true;
					var _didIteratorError8 = false;
					var _iteratorError8 = undefined;

					try {
						for (var _iterator8 = artifact.payment.tokens[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
							var token = _step8.value;

							this.addTokenRule(token);
						}
					} catch (err) {
						_didIteratorError8 = true;
						_iteratorError8 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion8 && _iterator8.return) {
								_iterator8.return();
							}
						} finally {
							if (_didIteratorError8) {
								throw _iteratorError8;
							}
						}
					}
				}
				if (artifact.payment.addresses) {
					var _iteratorNormalCompletion9 = true;
					var _didIteratorError9 = false;
					var _iteratorError9 = undefined;

					try {
						for (var _iterator9 = artifact.payment.addresses[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
							var address = _step9.value;

							this.addSinglePaymentAddress(address.token, address.address);
						}
					} catch (err) {
						_didIteratorError9 = true;
						_iteratorError9 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion9 && _iterator9.return) {
								_iterator9.return();
							}
						} finally {
							if (_didIteratorError9) {
								throw _iteratorError9;
							}
						}
					}
				}
				if (artifact.payment.retailer) {
					this.setRetailerCut(artifact.payment.retailer);
				}
				if (artifact.payment.promoter) {
					this.setPromoterCut(artifact.payment.promoter);
				}
				if (artifact.payment.maxdisc) {
					this.setMaxDiscount(artifact.payment.maxdisc);
				}
			}
		}
	}, {
		key: 'import042',
		value: function import042(artifact) {
			if (artifact.floAddress) {
				this.setMainAddress(artifact.floAddress);
			}
			if (artifact.timestamp) {
				this.setTimestamp(artifact.timestamp);
			}
			if (artifact.type) {
				this.setType(artifact.type);
			}
			if (artifact.subtype) {
				this.setSubtype(artifact.subtype);
			}
			if (artifact.signature) {
				this.setSignature(artifact.signature);
			}
			if (artifact.info) {
				if (artifact.info.title) {
					this.setTitle(artifact.info.title);
				}
				if (artifact.info.description) {
					this.setDescription(artifact.info.description);
				}
				if (artifact.info.year) {
					this.setYear(artifact.info.year);
				}
				if (artifact.info.tags) {
					this.setTags(artifact.info.tags);
				}
				if (artifact.info.nsfw) {
					this.setNSFW(artifact.info.nsfw);
				}
			}

			if (artifact.details) {
				for (var key in artifact.details) {
					if (artifact.details.hasOwnProperty(key)) {
						this.setDetail(key, artifact.details[key]);
					}
				}
			}

			if (artifact.storage) {
				if (artifact.storage.network) {
					this.setNetwork(artifact.storage.network);
				}
				if (artifact.storage.location) {
					this.setLocation(artifact.storage.location);
				}
				if (artifact.storage.files) {
					var _iteratorNormalCompletion10 = true;
					var _didIteratorError10 = false;
					var _iteratorError10 = undefined;

					try {
						for (var _iterator10 = artifact.storage.files[Symbol.iterator](), _step10; !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
							var file = _step10.value;

							this.addFile(file);
						}
					} catch (err) {
						_didIteratorError10 = true;
						_iteratorError10 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion10 && _iterator10.return) {
								_iterator10.return();
							}
						} finally {
							if (_didIteratorError10) {
								throw _iteratorError10;
							}
						}
					}
				}
			}

			if (artifact.payment) {
				if (artifact.payment.fiat) {
					this.setPaymentFiat(artifact.payment.fiat);
				}
				if (artifact.payment.scale) {
					this.setPaymentScale(artifact.payment.scale);
				}
				if (artifact.payment.sugTip) {
					this.setSuggestedTip(artifact.payment.sugTip);
				}
				if (artifact.payment.tokens && Array.isArray(artifact.payment.tokens)) {
					var _iteratorNormalCompletion11 = true;
					var _didIteratorError11 = false;
					var _iteratorError11 = undefined;

					try {
						for (var _iterator11 = artifact.payment.tokens[Symbol.iterator](), _step11; !(_iteratorNormalCompletion11 = (_step11 = _iterator11.next()).done); _iteratorNormalCompletion11 = true) {
							var token = _step11.value;

							this.addTokenRule(token);
						}
					} catch (err) {
						_didIteratorError11 = true;
						_iteratorError11 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion11 && _iterator11.return) {
								_iterator11.return();
							}
						} finally {
							if (_didIteratorError11) {
								throw _iteratorError11;
							}
						}
					}
				}
				if (artifact.payment.addresses) {
					var _iteratorNormalCompletion12 = true;
					var _didIteratorError12 = false;
					var _iteratorError12 = undefined;

					try {
						for (var _iterator12 = artifact.payment.addresses[Symbol.iterator](), _step12; !(_iteratorNormalCompletion12 = (_step12 = _iterator12.next()).done); _iteratorNormalCompletion12 = true) {
							var address = _step12.value;

							this.addSinglePaymentAddress(address.token, address.address);
						}
					} catch (err) {
						_didIteratorError12 = true;
						_iteratorError12 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion12 && _iterator12.return) {
								_iterator12.return();
							}
						} finally {
							if (_didIteratorError12) {
								throw _iteratorError12;
							}
						}
					}
				}
				if (artifact.payment.retailer) {
					this.setRetailerCut(artifact.payment.retailer);
				}
				if (artifact.payment.promoter) {
					this.setPromoterCut(artifact.payment.promoter);
				}
				if (artifact.payment.maxdisc) {
					this.setMaxDiscount(artifact.payment.maxdisc);
				}
			}
		}
	}, {
		key: 'getMultiparts',
		value: function getMultiparts() {
			var jsonString = this.toString();

			if (jsonString.length > FLODATA_MAX_LEN || this.fromMultipart) {
				var exactMatch = false;

				var oldArtifact = new Artifact();
				oldArtifact.fromMultiparts(this.Multiparts);

				if (oldArtifact.toString() === jsonString) exactMatch = true;

				if (!exactMatch) {
					this.Multiparts = [];

					var chunks = [];
					while (jsonString.length > CHOP_MAX_LEN) {
						chunks[chunks.length] = jsonString.slice(0, CHOP_MAX_LEN);
						jsonString = jsonString.slice(CHOP_MAX_LEN);
					}
					chunks[chunks.length] = jsonString;

					for (var c in chunks) {
						var mp = new _Multipart2.default();

						mp.setPartNumber(c);
						mp.setTotalParts(chunks.length);
						mp.setPublisherAddress(this.getMainAddress());
						mp.setChoppedStringData(chunks[c]);

						// If we are the first multipart, then sign ourself
						if (c === 0) {
							// @TODO: Implement multipart signing
							mp.sign();
						}

						this.Multiparts.push(mp);
					}
				}

				return this.Multiparts;
			} else {
				// Too short to be a multipart!
				return [];
			}
		}
	}, {
		key: 'fromMultiparts',
		value: function fromMultiparts(multipartArray) {
			if (Array.isArray(multipartArray)) {
				for (var part in multipartArray) {
					if (multipartArray[part] instanceof _Multipart2.default) {
						this.Multiparts[part] = multipartArray[part];
					} else {
						var mp = new _Multipart2.default();
						mp.fromString(multipartArray[part]);
						this.Multiparts.push(mp);
					}
				}

				if (Array.isArray(this.Multiparts)) {
					this.Multiparts.sort(function (a, b) {
						return a.getPartNumber() - b.getPartNumber();
					});
				}

				var jsonString = "";

				var _iteratorNormalCompletion13 = true;
				var _didIteratorError13 = false;
				var _iteratorError13 = undefined;

				try {
					for (var _iterator13 = this.Multiparts[Symbol.iterator](), _step13; !(_iteratorNormalCompletion13 = (_step13 = _iterator13.next()).done); _iteratorNormalCompletion13 = true) {
						var multiP = _step13.value;

						jsonString += multiP.getChoppedStringData();
					}
				} catch (err) {
					_didIteratorError13 = true;
					_iteratorError13 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion13 && _iterator13.return) {
							_iterator13.return();
						}
					} finally {
						if (_didIteratorError13) {
							throw _iteratorError13;
						}
					}
				}

				try {
					this.fromJSON(JSON.parse(jsonString));
					if (this.Multiparts[0] && this.Multiparts[0].getTXID() !== "") {
						this.setTXID(this.Multiparts[0].getTXID());
					}
					this.fromMultipart = true;
				} catch (e) {
					return { success: false, message: "Unable to parse from JSON!", error: e };
				}
			} else {
				return { success: false, message: "You must pass an array!" };
			}
		}
	}, {
		key: 'capitalizeFirstLetter',
		value: function capitalizeFirstLetter(string) {
			return string.charAt(0).toUpperCase() + string.slice(1).toLowerCase();
		}
	}, {
		key: 'getClassName',
		value: function getClassName() {
			return "Artifact";
		}
	}]);

	return Artifact;
}();